// Copyright 2015-2020 SWIM.AI inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package swim.http.header;

import org.testng.annotations.Test;
import swim.http.Http;
import swim.http.HttpAssertions;
import swim.http.HttpHeader;
import static swim.http.HttpAssertions.assertWrites;

public class SecWebSocketExtensionsSpec {

  public void assertParses(String string, HttpHeader header) {
    HttpAssertions.assertParses(Http.standardParser().headerParser(), string, header);
  }

  @Test
  public void parseSecWebSocketExtensionsHeaders() {
    assertParses("Sec-WebSocket-Extensions: foo",
        SecWebSocketExtensions.from("foo"));
    assertParses("Sec-WebSocket-Extensions: bar;baz=2",
        SecWebSocketExtensions.from("bar; baz=2"));
    assertParses("Sec-WebSocket-Extensions: foo , bar ; baz = 2",
        SecWebSocketExtensions.from("foo", "bar; baz=2"));
    assertParses("Sec-WebSocket-Extensions: mux; max-channels=\"inf\"; flow-control, deflate-stream",
        SecWebSocketExtensions.from("mux; max-channels=inf; flow-control", "deflate-stream"));
  }

  @Test
  public void writeSecWebSocketExtensionsHeaders() {
    assertWrites(SecWebSocketExtensions.from("foo"),
        "Sec-WebSocket-Extensions: foo");
    assertWrites(SecWebSocketExtensions.from("bar; baz=2"),
        "Sec-WebSocket-Extensions: bar; baz=2");
    assertWrites(SecWebSocketExtensions.from("foo", "bar; baz=2"),
        "Sec-WebSocket-Extensions: foo, bar; baz=2");
    assertWrites(SecWebSocketExtensions.from("mux; max-channels=\"inf\"; flow-control", "deflate-stream"),
        "Sec-WebSocket-Extensions: mux; max-channels=inf; flow-control, deflate-stream");
  }

}
